@@ -454,7 +454,9 @@ class SamApi(SamResourceMacro):
         'GatewayResponses': PropertyType(False, is_type(dict)),
         'AccessLogSetting': PropertyType(False, is_type(dict)),
         'CanarySetting': PropertyType(False, is_type(dict)),
-        'TracingEnabled': PropertyType(False, is_type(bool))
+        'TracingEnabled': PropertyType(False, is_type(bool)),
+        'OpenApiVersion': PropertyType(False, is_str()),
+        'Models': PropertyType(False, is_type(dict))
     }
 
     referable_properties = {
@@ -492,7 +494,9 @@ class SamApi(SamResourceMacro):
                                      canary_setting=self.CanarySetting,
                                      tracing_enabled=self.TracingEnabled,
                                      resource_attributes=self.resource_attributes,
-                                     passthrough_resource_attributes=self.get_passthrough_resource_attributes())
+                                     passthrough_resource_attributes=self.get_passthrough_resource_attributes(),
+                                     open_api_version=self.OpenApiVersion,
+                                     models=self.Models)
 
         rest_api, deployment, stage, permissions = api_generator.to_cloudformation()
 
